﻿/// <reference path="../Libs/CrmMessages/Sdk.RetrieveMetadataChanges.js" />
var ace = window.ace || { __namespace: true };
ace.Global = ace.Global || { __namespace: true };

(function ()
{
    this.GetContext = function ()
    {
        /// <summary>
        /// Returns the context, independent if it's called from Web Resource or a Form.
        /// </summary>
        /// <returns type="Xrm.Page.Context">Client-side context</returns>
        var errorMessage = "Context is not available.";
        if (typeof GetGlobalContext != "undefined")
        { return GetGlobalContext(); }
        else
        {
            if (typeof Xrm != "undefined")
            {
                return Xrm.Page.context;
            }
            else
            { throw new Error(errorMessage); }
        }
    }

    this.GetMetadataPropertiesByTypeCode = function (objectTypeCode, metadataProperties)
    {
        /// <summary>
        /// Retruns the entity metadata properties for specified entity type code
        /// </summary>
        /// <param name="objectTypeCode">number, being the entity type code</param>
        /// <param name="metadataProperties">array of property names, it's best to take them from Sdk.Mdq.EntityMetadataProperties</param>
        /// <returns type="$.Deferred">Deferred result. Success callback will be called with the parameter of an object having all the selected attributes</returns>
        var deferredResult = $.Deferred();

        //Set entity filter to return only the entity with passed objectTypeCode
        var entityFilter = new Sdk.Mdq.MetadataFilterExpression(Sdk.Mdq.LogicalOperator.And);
        entityFilter.addCondition(Sdk.Mdq.SearchableEntityMetadataProperties.ObjectTypeCode, Sdk.Mdq.MetadataConditionOperator.Equals, parseInt(objectTypeCode));

        //Set entity properties to only return LogicalName
        var entityProperties = new Sdk.Mdq.MetadataPropertiesExpression(false, metadataProperties);

        //Instantiate entity query
        var query = new Sdk.Mdq.EntityQueryExpression(entityFilter, entityProperties);

        //Execute the request
        Sdk.jQ.execute(new Sdk.RetrieveMetadataChangesRequest(query))
            .done(function (result)
            {
                if (result)
                {
                    var md = result.getEntityMetadata();
                    if (md && md[0])
                    {
                        deferredResult.resolve(md[0]);
                    }
                    else
                        deferredResult.reject("Entity not found.");
                }
                else
                    deferredResult.reject("Empty response from Soap endpoint.");
            })
            .fail(function (error)
            {
                deferredResult.reject(error);
            });

        return deferredResult.promise();
    };

    this.GetOptionSet = function (fieldName, entityName)
    {
        /// <summary>
        /// Fetches the optionset from the Metadata.
        /// </summary>
        /// <param name="fieldName">Field name, for which the optionset should be obtained (doesn't matter if global or local)</param>
        /// <param name="entityName">Entity name</param>
        /// <returns type="$.Deferred">Deferred result. Success callback will be called with the option set as the first parameter.</returns>
        var deferredResult = $.Deferred();

        //Set entity filter to return only the entity with passed entity name
        var entityFilter = new Sdk.Mdq.MetadataFilterExpression();
        entityFilter.addCondition(Sdk.Mdq.SearchableEntityMetadataProperties.LogicalName, Sdk.Mdq.MetadataConditionOperator.Equals, entityName);

        //Set entity properties to only return Attributes
        var entityProperties = new Sdk.Mdq.MetadataPropertiesExpression(false, [Sdk.Mdq.EntityMetadataProperties.Attributes]);

        //Set attribute filter to just include selected field
        var attributeFilter = new Sdk.Mdq.MetadataFilterExpression();
        attributeFilter.addCondition(Sdk.Mdq.SearchableAttributeMetadataProperties.LogicalName, Sdk.Mdq.MetadataConditionOperator.Equals, fieldName);

        //Set attribute properties to only include the attribute type and schema name
        var attributeProperties = new Sdk.Mdq.MetadataPropertiesExpression(false, [Sdk.Mdq.AttributeMetadataProperties.OptionSet]);

        //Instantiate entity query
        var query = new Sdk.Mdq.EntityQueryExpression(entityFilter, entityProperties, new Sdk.Mdq.AttributeQueryExpression(attributeFilter, attributeProperties));

        //Execute the request
        Sdk.jQ.execute(new Sdk.RetrieveMetadataChangesRequest(query))
            .done(function (result)
            {
                if (result)
                {
                    var md = result.getEntityMetadata();
                    if (md && md[0] && md[0].Attributes && md[0].Attributes[0] && md[0].Attributes[0].OptionSet)
                    {
                        deferredResult.resolve(md[0].Attributes[0].OptionSet);
                    }
                    else
                        deferredResult.reject("Optionset not found.");
                }
                else
                    deferredResult.reject("Empty response from Soap endpoint.");
            })
            .fail(function (error)
            {
                deferredResult.reject(error);
            });

        return deferredResult.promise();
    };

    this.CreateCopyOfObject = function (entityToCopy, newAttributes)
    {
        /// <summary>
        /// Creates the copy of object and returns the deferred object. Callback function will receive the id of newly created entity. Old entity remains intact.
        /// </summary>
        /// <param name="entityToCopy" type="Sdk.Entity">Sdk.Entity instance to copy</param>
        /// <param name="newAttributes">This should have the following construction: {attrname: {value: value, type: type}}, where types are the types of fields from Sdk. namespace</param>
        /// <returns type="$.Deferred">Deferred result. Success callback will be called with the id of newly created entity.</returns>

        var newEntity = new Sdk.Entity(entityToCopy.getType());
        var newId = ace.Global.Guid.Empty;
        newEntity.setId(newId);

        if (!newAttributes)
            newAttributes = {};

        // We need to remove some values, as crm is not gonna accept them:(
        entityToCopy.getAttributes().forEach(function (attr, i)
        {
            /// <param name="attr" type="Sdk.AttributeBase"></param>

            var attrname = attr.getName().toLowerCase();

            if (attrname == "ownerid")
            {
                //todo owner should be set here to current user
            }
            else if (attr.getType() == Sdk.ValueType.entityCollection)
            {
                //attr is now Sdk.PartyList, and it's value is EntityCollection
                var partyEntities = attr.getValue();

                if (partyEntities && partyEntities.getCount() > 0)
                {
                    partyEntities.getEntities().forEach(function (activityParty, index)
                    {
                        /// <param name="activityParty" type="Sdk.Entity"></param>
                        activityParty.setValue("activitypartyid", ace.Global.Guid.New());
                    });
                    var partyAttribute = new Sdk.PartyList(attr.getName(), partyEntities);
                    newEntity.addAttribute(partyAttribute);
                }
            }
            else if (attrname != "activityid" && attrname != entityToCopy.getType().toLowerCase() + "id")
            {
                var attributeClass = ace.Global.GetSdkTypeFromAttributeType(attr.getType());
                var newAttribute = new attributeClass(attr.getName(), attr.getValue());
                newEntity.addAttribute(newAttribute);
            }
        });

        //We need to also add newAttributes, also because potentially the old entity didn't have them
        for (var attrName in newAttributes)
        {
            //If it was already added, this method will set the new value
            newEntity.addAttribute(new newAttributes[attrName].type(attrName, newAttributes[attrName].value));
        }

        return Sdk.jQ.create(newEntity);
    };

    this.GetSdkTypeFromAttributeType = function (attrType)
    {
        /// <summary>
        /// Used internally at the moment, but it returns the Sdk.Type based on the string returned from Sdk.getAttribute().getType(). For example from 
        /// Sdk.ValueType.string we get Sdk.String. The latter one is required to instantiate the new attribute of this type.
        /// </summary>
        /// <param name="attrType">String containing Sdk.ValueType, most likely obtained from attribute.getType()</param>
        /// <returns type="">the Sdk.Type, which can be used to instantiate the new attribute</returns>
        switch (attrType)
        {
            case Sdk.ValueType.string:
                return Sdk.String;
                break;
            case Sdk.ValueType.guid:
                return Sdk.Guid;
                break;
            case Sdk.ValueType.money:
                return Sdk.Money;
                break;
            case Sdk.ValueType.long:
                return Sdk.Long;
                break;
            case Sdk.ValueType.decimal:
                return Sdk.Decimal;
                break;
            case Sdk.ValueType.double:
                return Sdk.Double;
                break;
            case Sdk.ValueType.int:
                return Sdk.Int;
                break;
            case Sdk.ValueType.optionSet:
                return Sdk.OptionSet;
                break;
            case Sdk.ValueType.boolean:
                return Sdk.Boolean;
                break;
            case Sdk.ValueType.dateTime:
                return Sdk.DateTime;
                break;
            case Sdk.ValueType.entityReference:
                return Sdk.Lookup;
                break;
            case Sdk.ValueType.booleanManagedProperty:
                return Sdk.BooleanManagedProperty;
                break;
            case Sdk.ValueType.entityCollection:
                return Sdk.PartyList;
                break;
            default:
                throw new Error("Sdk unimplemented type found in getTypedValue function");
        }
    }

    this.GetEnumDisplayNameFromNumber = function (number, enumDefinition)
    {
        /// <summary>
        /// Returns the display name based on the value of the enum
        /// </summary>
        /// <param name="number">Enums value</param>
        /// <param name="enumDefinition">enums class</param>
        /// <returns type="string">Display text of the enum with the specified value</returns>
        return enumDefinition._NamesByValues[number];
    };

    //this one is here just because from ribbon it is not possible to open the form this way (in same window, without reloading)
    this.OpenForm = function (entityName, id, parameters, newWindow)
    {
        if (newWindow)
        {
            var url = Xrm.Page.context.getClientUrl();
            url += "/main.aspx?etn=" + entityName + "&pagetype=entityrecord";
            if (parameters)
            {
                var parametersInUrl = [];

                for (key in parameters)
                {
                    parametersInUrl.push(key + "=" + parameters[key]);
                }

                url += "&extraqs=" + encodeURIComponent(parametersInUrl.join("&"));
            }
            if (id)
                url += "&id={" + id + "}"
            window.open(url, '_blank', 'location=0,menubar=0,scrollbars=0,status=0,titlebar=0,toolbar=0,resizable=yes', true);
        }
        else
        {
            Xrm.Utility.openEntityForm(entityName, id, parameters);
        }
    };

    //this one is here just because from ribbon it is not possible to open web resource with parameters
    this.OpenWebresource = function (webresourceName, parameters, width, height)
    {
        //if array or object passed - convert it to string suitable for url
        if (parameters && typeof parameters !== 'string' && !(parameters instanceof String))
        {
            var parametersInUrl = [];

            for (key in parameters)
            {
                parametersInUrl.push(key + "=" + parameters[key]);
            }

            parameters = parametersInUrl.join("&");
        }

        Xrm.Utility.openWebResource(webresourceName, encodeURIComponent(parameters), width, height);
    };

    this.GetQueryStringParametersAsArray = function (parametersString)
    {
        /// <summary>
        /// Extracts the parameters from current location's query string. If parameter is passed, it is used instead.
        /// </summary>
        /// <param name="parametersString" optional="true"></param>
        var match,
            pl = /\+/g,  // Regex for replacing addition symbol with a space
            search = /([^&=]+)=?([^&]*)/g,
            decode = function (s) { return decodeURIComponent(s.replace(pl, " ")); },
            query = parametersString;

        if (!query)
        {
            query = window.location.search.substring(1);
        }

        urlParams = {};
        while (match = search.exec(query))
            urlParams[decode(match[1])] = decode(match[2]);
        return urlParams;
    };

    this.DisableAllFieldsOntheForm = function (disabled)
    {
        /// <summary>
        /// Disables all the fields on the current form
        /// </summary>
        /// <param name="disabled">true to disable, false to enable</param>
        if (typeof disabled == 'undefined')
            disabled = true;
        var allControls = Xrm.Page.getControl();
        allControls.forEach(function (control)
        {
            control.setDisabled(disabled);
        });
    };

    this.AreArraysEqual = function (array1, array2)
    {
        /// <summary>
        /// Compares if two arrays the same elements
        /// </summary>
        /// <param name="array1">First array</param>
        /// <param name="array2">Second array</param>
        /// <returns type="">true if equal, false otherwise</returns>
        // if the other array is a falsy value, return
        if (!array1 || !array2)
            return false;

        // compare lengths - can save a lot of time 
        if (array1.length != array2.length)
            return false;

        for (var i = 0, l = this.length; i < l; i++)
        {
            // Check if we have nested arrays
            if (array1[i] instanceof Array && array2[i] instanceof Array)
            {
                // recurse into the nested arrays
                if (!array1[i].equals(array2[i]))
                    return false;
            }
            else if (array1[i] != array2[i])
            {
                // Warning - two different object instances will never be equal: {x:20} != {x:20}
                return false;
            }
        }
        return true;
    };

    this.hideBPF = function (hide)
    {
        if(document.getElementById("processControlContainer"))
        {
            if (hide)
                document.getElementById("processControlContainer").style.display = 'none';
            else
                document.getElementById("processControlContainer").style.display = 'block';
        }
    }

    this.hideBPFBack = function (hide)
    {
        if(document.getElementById("stageBackActionContainer"))
        {
            if (hide)
                document.getElementById("stageBackActionContainer").style.display = 'none';
            else
                document.getElementById("stageBackActionContainer").style.display = 'block';
        }
    }

    this.hideBPFAdvance = function (hide)
    {
        if(document.getElementById("stageAdvanceActionContainer"))
        {
            if (hide)
                document.getElementById("stageAdvanceActionContainer").style.display = 'none';
            else
                document.getElementById("stageAdvanceActionContainer").style.display = 'block';
        }
    }

    this.changeNextPhaseText = function (newText)
    {
        var nextPhaseNodes = $("#stageAdvanceActionContainer .stageActionText").contents().filter(function ()
        {
            return this.nodeType === 3; //Node.TEXT_NODE
        });
        if (nextPhaseNodes && nextPhaseNodes.length > 1)
            nextPhaseNodes[1].nodeValue = newText;
    }

    this.hideProcessesInMenu = function (toHide)
    {
        if (document.querySelectorAll && toHide && toHide.length)
        {
            var switcher = document.getElementById("processSwitcherFlyout");
            if (switcher)
            {
                var selector = "";
                for (var i = 0; i < toHide.length; i++)
                    selector += "div.ms-crm-ProcessSwitcher-ProcessTitle[title='" + toHide[i] + "'],";

                selector = selector.substring(0, selector.length - 1);

                var result = switcher.querySelectorAll(selector);
                if (result && result.length)
                {
                    for (var i = 0; i < result.length; i++)
                    {
                        if (result[i].parentNode)
                            result[i].parentNode.style.display = 'none';
                    }
                }
                else
                {
                    setTimeout(function () { ace.Global.hideProcessesInMenu(toHide); }, 250);
                }
            }
            else
            {
                setTimeout(function () { ace.Global.hideProcessesInMenu(toHide); }, 250);
            }
        }
    };

    this.getExceptionText = function (errorObject)
    {
        var result = "Fehler aufgetreten: ";
        if (errorObject && errorObject.responseJSON && errorObject.responseJSON.Message)
            result = errorObject.responseJSON.Message;
        else if (errorObject && errorObject.responseText)
            result = errorObject.responseText;
        else
            result = errorObject;
        return result;
    }

    this.Guid = {};
    this.Dialog = {};
}).call(ace.Global);

(function ()
{
    this.GetProperGuidForSdkSoap = function (guid)
    {
        /// <summary>
        /// Converts the Guid to the form accepted by Sdk.Soap
        /// </summary>
        /// <param name="guid">string containing guid in any form, with braces or not, etc.</param>
        /// <returns type="">string containing guid suitable for Sdk.Soap queries.</returns>
        return guid.replace(/([{}])/gi, "");
    };

    //Returns the empty guid
    this.Empty = '00000000-0000-0000-0000-000000000000';

    this.New = function ()
    {
        /// <summary>
        /// Retruns the newly generated Guid
        /// </summary>
        /// <returns type="">New pseudo-random Guid</returns>
        var d = new Date().getTime();
        var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c)
        {
            var r = (d + Math.random() * 16) % 16 | 0;
            d = Math.floor(d / 16);
            return (c == 'x' ? r : (r & 0x7 | 0x8)).toString(16);
        });
        return uuid;
    };
}).call(ace.Global.Guid);

(function ()
{

    this.Open = function (webresourceUrl, width, height, params, callback)
    {
        /// <summary>
        /// Opens a webresource in a default CRM dialog box. UNSUPPORTED
        /// </summary>
        /// <param name="webresourceUrl">relative url to the webresource</param>
        /// <param name="width">Width of the dialog</param>
        /// <param name="height">Height of the dialog</param>
        /// <param name="callback">Function to call when dialog is closed. It will get a return value as first parameter. If not passed, then this will be executed Synchronously.</param>  	

        var DialogOption = new Xrm.DialogOptions;
        DialogOption.width = width;
        DialogOption.height = height;
        Xrm.Internal.openDialog(webresourceUrl,
                                DialogOption,
                                params, null,
                                callback);
    };

    this.Close = function (returnValue)
    {
        /// <summary>
        /// Closes current Crm dialog. Should be called from within the dialog. UNSUPPORTED
        /// </summary>
        /// <param name="returnValue">Value to call the callback with</param>
        //Warning: this one calls the callback immediately, not only setting the value
        if (returnValue)
            Mscrm.Utilities.setReturnValue(returnValue);

        //this is how MS does this... actually they have a timeout of 5ms set.
        //In case of problems rethink this... It would be best to call it after callback finished execution
        window.setTimeout(Mscrm.Utilities.closeCurrentWindow, 50);
    };

    this.GetArguments = function ()
    {
        /// <summary>
        /// Gets the arguments passed to the dialog. UNSUPPORTED
        /// </summary>

        //this is the method that ms uses.
        if (getDialogArguments)
            return getDialogArguments();
    };

    this.ShowProgress = function (show)
    {
        if (document.querySelector)
        {
            var progressDiv = document.querySelector(".dlgProgress");
            if (progressDiv)
            {
                if (show)
                    progressDiv.style.display = 'block'
                else
                    progressDiv.style.display = 'none'
            }
        }
    }
}).call(ace.Global.Dialog);

if (window.Sdk)
{
    Sdk.Entity.prototype.getValueOrNull = function (attrName)
    {
        try
        {
            return this.getValue(attrName);
        }
        catch (e)
        {
            return null;
        }
    };
}