param($installPath, $toolsPath, $package, $project)


function Set-PackageToBeDevelopmentDependency($PackageId, $PackageVersion, $ProjectDirectoryPath, $LibFolder, $PackageInstallPath)
{
    function Get-XmlNamespaceManager([xml]$XmlDocument, [string]$NamespaceURI = "")
    {
        # If a Namespace URI was not given, use the Xml document's default namespace.
	    if ([string]::IsNullOrEmpty($NamespaceURI)) { $NamespaceURI = $XmlDocument.DocumentElement.NamespaceURI }	
	
	    # In order for SelectSingleNode() to actually work, we need to use the fully qualified node path along with an Xml Namespace Manager, so set them up.
	    [System.Xml.XmlNamespaceManager]$xmlNsManager = New-Object System.Xml.XmlNamespaceManager($XmlDocument.NameTable)
	    $xmlNsManager.AddNamespace("ns", $NamespaceURI)
        return ,$xmlNsManager		# Need to put the comma before the variable name so that PowerShell doesn't convert it into an Object[].
    }

    function Get-FullyQualifiedXmlNodePath([string]$NodePath, [string]$NodeSeparatorCharacter = '.')
    {
        return "/ns:$($NodePath.Replace($($NodeSeparatorCharacter), '/ns:'))"
    }

    function Get-XmlNodes([xml]$XmlDocument, [string]$NodePath, [string]$NamespaceURI = "", [string]$NodeSeparatorCharacter = '.')
    {
	    $xmlNsManager = Get-XmlNamespaceManager -XmlDocument $XmlDocument -NamespaceURI $NamespaceURI
	    [string]$fullyQualifiedNodePath = Get-FullyQualifiedXmlNodePath -NodePath $NodePath -NodeSeparatorCharacter $NodeSeparatorCharacter

	    # Try and get the nodes, then return them. Returns $null if no nodes were found.
	    $nodes = $XmlDocument.SelectNodes($fullyQualifiedNodePath, $xmlNsManager)
	    return $nodes
    }

    function Replace-TTFileTokens([string]$ttFilePath, [string]$TokenName, [string]$TokenReplacement)
    {
       $fileContent = [System.IO.File]::ReadAllText($ttFilePath);
       $replacedContent = $fileContent.Replace($TokenName, $TokenReplacement);
       [System.IO.File]::WriteAllText($ttFilePath, $replacedContent);
    }

    # Get the path to the project's packages.config file.
    Write-Output "Project directory is '$ProjectDirectoryPath'."
    $packagesConfigFilePath = Join-Path $ProjectDirectoryPath "packages.config"    
    
    # If we found the packages.config file, try and update it.
    if (Test-Path -Path $packagesConfigFilePath)
    {
        Write-Output "Found packages.config file at '$packagesConfigFilePath'."
    
        # Load the packages.config xml document and grab all of the <package> elements.
        $xmlFile = New-Object System.Xml.XmlDocument
        $xmlFile.Load($packagesConfigFilePath)
        $packageElements = Get-XmlNodes -XmlDocument $xmlFile -NodePath "packages.package"

        Write-Output "Packages.config contents before modification are:`n$($xmlFile.InnerXml)"

        if (!($packageElements))
        {
            Write-Output "Could not find any <package> elements in the packages.config xml file '$packagesConfigFilePath'."
            return
        }
        
        Write-Output "Libfolder: '$LibFolder'"
        Write-Output "ProjectDirectoryPath: '$ProjectDirectoryPath'"
        Write-Output "PackageInstallPath: '$PackageInstallPath'"
        
        if($ProjectDirectoryPath)
        {
            if (Test-Path -Path $ProjectDirectoryPath)
            {    
                Write-Output $PackageInstallPath
                $packagesPath = ([System.IO.Directory]::GetParent($PackageInstallPath))
                $tracerPackagePath = [System.IO.Directory]::GetDirectories($packagesPath, "ace.Tracer.TraceClient*", [System.IO.SearchOption]::AllDirectories)
                $xrmPackagePath = [System.IO.Directory]::GetDirectories($packagesPath, "Microsoft.CrmSdk.CoreAssemblies*", [System.IO.SearchOption]::AllDirectories)

                Write-Output $packagesPath

                $traceDlls = [System.IO.Directory]::GetFiles($tracerPackagePath, "ace.Tracer.TraceClient.dll", [System.IO.SearchOption]::AllDirectories)
                if($traceDlls)
                {
                   $traceDll = $traceDlls[0].Replace($packagesPath, "`$(SolutionDir)\packages")
                }
                $xrmsdkDlls = [System.IO.Directory]::GetFiles($xrmPackagePath, "Microsoft.Xrm.Sdk.dll", [System.IO.SearchOption]::AllDirectories)
                if($xrmsdkDlls)
                {
                   $xrmsdkDll = $xrmsdkDlls[0].Replace($packagesPath, "`$(SolutionDir)\packages")
                }
                
                Write-Output "Found Libs folder"
                $LibFolder = "`$(SolutionDir)\packages\" + $PackageId + "." + $PackageVersion + $LibFolder
                $codeGenDll = $LibFolder + "ace.CRM.CodeGenerator.dll"
                    
                $ttFiles = [System.IO.Directory]::GetFiles($ProjectDirectoryPath, "*.tt*", [System.IO.SearchOption]::AllDirectories)
                if($ttFiles)
                {
                    if ($traceDll)
                    {
                        $ttFiles | ForEach-Object { Replace-TTFileTokens $_ "`$(SolutionDir)Libs\ace.Tracer.TraceClient.dll" $traceDll }
                    }

                    $ttFiles | ForEach-Object { Replace-TTFileTokens $_ "`$(SolutionDir)Libs\ace.CRM.CodeGenerator.dll" $codeGenDll }

                    if ($xrmsdkDll)
                    {
                        $ttFiles | ForEach-Object { Replace-TTFileTokens $_ "`$(SolutionDir)Libs\CRM\Microsoft.Xrm.sdk.dll" $xrmsdkDll }
                    }
                }
            }
        }
    
        # Add the developmentDependency attribute to the NuGet package's entry.
        $packageElements | Where-Object { $_.id -eq $PackageId } | ForEach-Object { $_.SetAttribute("developmentDependency", "true") }

        # Save the packages.config file back now that we've changed it.
        $xmlFile.Save($packagesConfigFilePath)
    }
    # Else we coudn't find the packages.config file for some reason, so error out.
    else
    {
        Write-Output "Could not find packages.config file at '$packagesConfigFilePath'."
    }
    
    
}

$libRelativeFolder = "\lib\"

Set-PackageToBeDevelopmentDependency -PackageId $package.Id -PackageVersion $package.Version -ProjectDirectoryPath ([System.IO.Directory]::GetParent($project.FullName)) -LibFolder $libRelativeFolder -PackageInstallPath $installPath